module.exports = function(grunt) {
  var buildDir = './build';

  var banner =
    '/*!\n\
 * <%= pkg.upperName %> v<%= pkg.version %>\n\
 * <%= pkg.repository.url %>\n\
 *\n\
 * Released under the MIT license\n\
 */\n';

  function addSrcPrefix(files) {
    return files.map(file => {
      return 'src/' + file;
    });
  }

  var files = addSrcPrefix([
    'head.js',
    'common.js',
    'createLexer.js',
    'lexer.js',
    'parser.js',
    'codegen_common.js',
    'codegen_js.js',
    'crox_js.js',
    'codegen_php.js',
    'codegen_vm.js',
    'crox_extra.js',
    'tail.js',
  ]);

  var filesClean = addSrcPrefix([
    'head.js',
    'common.js',
    'createLexer.js',
    'lexer.js',
    'parser.js',
    'codegen_common.js',
    'codegen_js.js',
    'crox_js.js',
    'tail.js',
  ]);

  var files2 = addSrcPrefix([
    'head2.js',
    'common.js',
    'createLexer.js',
    'lexer.js',
    'parser2.js',
    'codeg2.js',
    'crox_js2.js',
    'tail2.js',
  ]);

  var pkg = grunt.file.readJSON('package.json');
  pkg.upperName = pkg.name.slice(0, 1).toUpperCase() + pkg.name.slice(1);

  grunt.initConfig({
    pkg: pkg,

    clean: {
      build: {
        src: buildDir,
      },
      options: {
        force: true,
      },
    },

    uglify: {
      options: {
        banner: banner,
      },
      dist_all: {
        files: {
          'build/<%= pkg.name %>-all-min.js': ['<%= concat.dist_all.dest %>'],
        },
      },
      dist_clean: {
        files: {
          'build/<%= pkg.name %>-min.js': ['<%= concat.dist_clean.dest %>'],
        },
      },
      dist_2: {
        files: {
          'build/<%= pkg.name %>2-min.js': ['<%= concat.dist_2.dest %>'],
        },
      },
    },

    concat: {
      options: {
        separator: '\n',
        banner: banner,
      },
      dist_all: {
        src: files,
        dest: buildDir + '/<%= pkg.name %>-all.js',
      },
      dist_clean: {
        src: filesClean,
        dest: buildDir + '/<%= pkg.name %>.js',
      },
      dist_2: {
        src: files2,
        dest: buildDir + '/<%= pkg.name %>2.js',
      },
    },

    version: {
      files: [buildDir + '/*.js'],
    },
  });

  grunt.registerMultiTask(
    'version',
    'replace version number in src',
    function() {
      var files = this.filesSrc;
      var version = pkg.version;

      console.log('[Crox] Current version of Crox is ' + version);
      files.forEach(function(f) {
        var content = grunt.file.read(f);
        if (content) {
          content = content.replace('%VERSION%', version);
          grunt.file.write(f, content);
        }
      });
    }
  );

  grunt.loadTasks('tasks');

  grunt.loadNpmTasks('grunt-contrib-uglify');
  grunt.loadNpmTasks('grunt-contrib-concat');
  grunt.loadNpmTasks('grunt-contrib-clean');

  grunt.registerTask('daily', ['addGitlab', 'pushGitlab:daily']);
  grunt.registerTask('deploy', ['addGitlab', 'pushGitlab:prod']);

  grunt.registerTask('build', ['clean', 'concat', 'uglify', 'version']);

  grunt.registerTask('default', ['build']);
};
